/*
 * readwrite.c
 *
 * Copyright (c) & Programmed 2010,2016 by NAKATA Hiroshi.
 */
#include <stdio.h>

#include <sys/types.h>

#ifndef TRUE
#define TRUE	(-1)
#define FALSE	0
#endif

/* write 16bit Little Endian data to file	-----------------------------*/
int write16bitint(FILE *fp, const int16_t data)
{
  if ( fputc( data & 0xff, fp) == EOF
       || fputc( (data >> 8) & 0xff, fp) == EOF ) {
    fprintf(stderr, "file write error\n" );
    return (FALSE);
  }
  return (TRUE);
}

/* write 24bit Little Endian data to file	-----------------------------*/
int write24bitint(FILE *fp, const int32_t data)
{
  if ( fputc( data & 0xff, fp) == EOF
       || fputc( (data >> 8) & 0xff, fp) == EOF
       || fputc( (data >> 16) & 0xff, fp) == EOF ) {
    fprintf(stderr, "file write error\n" );
    return (FALSE);
  }
  return (TRUE);
}

/* write 32bit Little Endian data to file	-----------------------------*/
int write32bitint(FILE *fp, const int32_t data)
{
  if ( fputc( data & 0xff, fp) == EOF
       || fputc( (data >> 8) & 0xff, fp) == EOF
       || fputc( (data >> 16) & 0xff, fp) == EOF
       || fputc( (data >> 24) & 0xff, fp) == EOF ) {
    fprintf(stderr, "file write error\n" );
    return (FALSE);
  }
  return (TRUE);
}

/* write 32bit float to file	-----------------------------*/
int writefloat(FILE *fp, const float data)
{
  if ( fwrite(&data, sizeof(float), 1, fp) != 1 ) {
    fprintf(stderr, "file write error\n" );
    return (FALSE);
  }
  return (TRUE);
}

/* write 64bit double to file	-----------------------------*/
int writedouble(FILE *fp, const double data)
{
  if ( fwrite(&data, sizeof(double), 1, fp) != 1 ) {
    fprintf(stderr, "file write error\n" );
    return (FALSE);
  }
  return (TRUE);
}

/* write 64bit Little Endian data to file	-----------------------------*/
int write64bitlong(FILE *fp, u_int64_t *p_int64)
{
  unsigned char buff[8];
  u_int64_t	ll = *p_int64;

  buff[0] = ll & 0xff;
  buff[1] = (ll >> 8) & 0xff;
  buff[2] = (ll >> 16) & 0xff;
  buff[3] = (ll >> 24) & 0xff;
  buff[4] = (ll >> 32) & 0xff;
  buff[5] = (ll >> 40) & 0xff;
  buff[6] = (ll >> 48) & 0xff;
  buff[7] = (ll >> 56) & 0xff;
  if ( fwrite(buff, 1, 8, fp) != 8 ) {
    fprintf(stderr, "fwrite() failed.\n" );
    return (FALSE);
  }

  return (TRUE);
}

/* read 32bit Little Endian data from file	-----------------------------*/
int read32bitint(FILE *fp, u_int32_t *p_int32)
{
  unsigned char buff[4];

  if ( fread(buff, 1, 4, fp) != 4 ) {
    fprintf(stderr, "fread() failed.\n" );
    return (FALSE);
  }

  *p_int32 = (buff[0] | (((int)buff[1]) << 8)
	      | (((int)buff[2]) << 16) | (((int)buff[3]) << 24));
  return (TRUE);
}

/* read 64bit Little Endian data from file	-----------------------------*/
int read64bitlong(FILE *fp, u_int64_t *p_int64)
{
  unsigned char buff[8];

  if ( fread(buff, 1, 8, fp) != 8 ) {
    fprintf(stderr, "fread() failed.\n" );
    return (FALSE);
  }

  *p_int64 = (buff[0] | (((int64_t)buff[1]) << 8)
	      | (((int64_t)buff[2]) << 16) | (((int64_t)buff[3]) << 24)
	      | (((int64_t)buff[4]) << 32) | (((int64_t)buff[5]) << 40)
	      | (((int64_t)buff[6]) << 48) | (((int64_t)buff[7]) << 56));
  return (TRUE);
}

/* read 16bit Big Endian data from file	-----------------------------*/
int read16bitBEint(FILE *fp, u_int16_t *p_int16)
{
  unsigned char buff[2];

  if ( fread(buff, 1, 2, fp) != 2 ) {
    fprintf(stderr, "fread() failed.\n" );
    return (FALSE);
  }

  *p_int16 = (buff[1] | ((int)buff[0]) << 8);
  return (TRUE);
}

/* read 32bit Big Endian data from file	-----------------------------*/
int read32bitBEint(FILE *fp, u_int32_t *p_int32)
{
  unsigned char buff[4];

  if ( fread(buff, 1, 4, fp) != 4 ) {
    fprintf(stderr, "fread() failed.\n" );
    return (FALSE);
  }

  *p_int32 = (buff[3] | (((int)buff[2]) << 8)
	      | (((int)buff[1]) << 16) | (((int)buff[0]) << 24));
  return (TRUE);
}

/* read 64bit Big Endian data from file	-----------------------------*/
int read64bitBElong(FILE *fp, u_int64_t *p_int64)
{
  unsigned char buff[8];

  if ( fread(buff, 1, 8, fp) != 8 ) {
    fprintf(stderr, "fread() failed.\n" );
    return (FALSE);
  }

  *p_int64 = (buff[7] | (((int64_t)buff[6]) << 8)
	      | (((int64_t)buff[5]) << 16) | (((int64_t)buff[4]) << 24)
	      | (((int64_t)buff[3]) << 32) | (((int64_t)buff[2]) << 40)
	      | (((int64_t)buff[1]) << 48) | (((int64_t)buff[0]) << 56));
  return (TRUE);
}

/* read 32bit float to file	-----------------------------*/
int readfloat(FILE *fp, float *p_data)
{
  if ( fread(p_data, sizeof(float), 1, fp) != 1 ) {
    fprintf(stderr, "file read error\n" );
    return (FALSE);
  }
  return (TRUE);
}

/* write 64bit double to file	-----------------------------*/
int readdouble(FILE *fp, double *p_data)
{
  if ( fread(p_data, sizeof(double), 1, fp) != 1 ) {
    fprintf(stderr, "file read error\n" );
    return (FALSE);
  }
  return (TRUE);
}
/* End of File	**************************************************************/
